<?php
/**
 * [TiSiE] RelativeTime
 *
 * Helper functions
 *
 * @package Tirt
 * @author Mathias Gelhausen <scripting@tisie.de>
 * @copyright 2011 [TiSiE]
 * @license http://www.gnu.org/licenses/old-licenses/gpl-2.0.html   GPLv2
 */


/**
 * Gets the date parts as array
 * 
 * @param string $date Date string (mysql DateTime format)
 * @return array
 */
function tirt_get_date_array($date=null)
{
    if (null === $date) {
        $date = current_time('mysql');
    }
    
    // $date is in format "YYYY-MM-DD HH:II:SS"
    list($datePart, $timePart) = explode(' ', $date, 2);
    list($year, $month, $day) = explode('-', $datePart, 3);
    list($hour, $minute, $second) = explode(':', $timePart, 3);
    
    return array(
        'year' => intval($year),
        'month' => intval($month),
        'day' => intval($day),
        'hour' => intval($hour),
        'minute' => intval($minute),
        'second' => intval($second)
    );
}

/**
 * Gets the date difference as array
 * 
 * @param string $date Date string (mysql DateTime format)
 * @return array Difference to current date 
 * @uses tirt_get_date_array()
 */
function tirt_get_array($date)
{
    global $tirt_units;
    
    $start = tirt_get_date_array($date);
    $stop = tirt_get_date_array();
    
    $values = array();
    $increment = 0;   
    
    /*
     * Nedded for the calculation.
     * Format is: unit => shift value
     * This is from second to year, because the algorithm might
     * need to increment the next "bigger" part. 
     */ 
    $tirt_units = array(
        'second' => 60,
        'minute' => 60,
        'hour' => 24,  
        'day' => array(31, -1, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31),
        'week' => false,
        'month' => 12,  
        'year' => false 
    );
    
    foreach ($tirt_units as $part => $shift) {
        if ('week'== $part) {
            // week is not in $values, so need to add it.
            $weeks = 0; 
            if ($values['day'] >= 7) {
                $weeks = floor($values['day'] / 7);
                $values['day'] -= $weeks * 7;
            }
            $values['week'] = $weeks;
            continue;
        } 

        if ($start[$part] + $increment > $stop[$part]) {
            // need to shift the start part (like in written subtraction)
            if ('day' == $part) {
              // day is special, because the shift value vary from month to month.
              $shift = $shift[$start['month'] - 1] == -1 // February depends on leap year.
                     ? ($start['year'] % 4 == 0 && ($start['year'] % 100 != 0 || $start['year'] % 400 == 0)
                        ? 29 : 28)
                     : $shift[$start['month'] -1];
            }
            $values[$part] = ($stop[$part] + $shift) - ($start[$part] + $increment);
            $increment = 1; // Must take the increment from the shifting to the next part.
        } else {
            $values[$part] = $stop[$part] - ($start[$part] + $increment);
            $increment = 0; // No shifting, no incrementing!
        }
        
    }  
    $values = array_reverse($values); // bring values in "right" order.
    return $values;
}

/**
 * Gets a relative time string
 * 
 * @param string $date Date string (mysql DateTime format)
 * @return string
 * @uses tirt_get_array()
 */
function tirt_get_string($date, $parts=2)
{
    global $tirt_units_map;
    $values = tirt_get_array($date);
    
    $i = 0; $outputChunks = array();
    foreach ($values as $unit => $value) {
        if (0 != $value) {
            $outputChunks[] = $value . ' ' . __($unit . ($value != 1 ? 's' : ''), 'tirt');
            $i += 1;
        }
        if ($i == $parts || ($i && 'week' != $unit && 0 == $value)) {
            // if there are already $parts parts or a value after the first part with a non-zero
            // value is "0". Week is handled special, because strings like
            // "3 months and 4 days" also makes sense.
            break;
        }
    }
    $output = '';
    if ($lastChunk = array_pop($outputChunks)) {
        if (count($outputChunks)) {
            $output .= implode(', ', $outputChunks) . ' ' . __('and', 'tirt') . ' ';
        }
        $output .= $lastChunk;
    } else {
        $ouput .= __('less than a second', 'tirt');
    }
    
    return $output;
}